@@ -3,15 +3,19 @@ extern crate chrono;
 extern crate predicates;
 extern crate tempfile;
 
+use crate::predicates::ends_with_pkg_version;
 use crate::predicates::file;
 
 use assert_cmd::assert::IntoOutputPredicate;
 use assert_cmd::Command;
+#[cfg(have_cairo_pdf)]
 use chrono::{TimeZone, Utc};
-use predicate::str::*;
+use predicates::boolean::*;
 use predicates::prelude::*;
+use predicates::str::*;
 use std::path::Path;
 use tempfile::Builder;
+use url::Url;
 
 // What should be tested here?
 // The goal is to test the code in rsvg-convert, not the entire library.
@@ -19,7 +23,7 @@ use tempfile::Builder;
 //  - command-line options that affect size (width, height, zoom, resolution) ✔
 //  - pixel dimensions of the output (should be sufficient to do that for PNG) ✔
 //  - limit on output size (32767 pixels) ✔
-//  - output formats (PNG, PDF, PS, EPS, SVG), okay to ignore XML and recording ✔
+//  - output formats (PNG, PDF, PS, EPS, SVG) ✔
 //  - multi-page output (for PDF) ✔
 //  - output file option ✔
 //  - SOURCE_DATA_EPOCH environment variable for PDF output ✔
@@ -32,31 +36,25 @@ use tempfile::Builder;
 struct RsvgConvert {}
 
 impl RsvgConvert {
-    fn binary_location() -> &'static Path {
-        match option_env!("LIBRSVG_BUILD_DIR") {
-            Some(dir) => Path::new(dir),
-            None => Path::new(env!("CARGO_MANIFEST_DIR")).parent().unwrap(),
-        }
-    }
-
     fn new() -> Command {
-        let path = Self::binary_location().join("rsvg-convert");
-        let mut command = Command::new(path);
+        let mut command = Command::cargo_bin("rsvg-convert").unwrap();
         command.env_clear();
         command
     }
 
-    fn new_with_input(input: &Path) -> Command {
+    fn new_with_input<P>(file: P) -> Command
+    where
+        P: AsRef<Path>,
+    {
         let mut command = RsvgConvert::new();
-        match command.pipe_stdin(input) {
+        match command.pipe_stdin(&file) {
             Ok(_) => command,
-            Err(e) => panic!("Error opening file '{}': {}", input.display(), e),
+            Err(e) => panic!("Error opening file '{}': {}", file.as_ref().display(), e),
         }
     }
 
-    fn accepts_option(option: &str) {
-        let input = Path::new("fixtures/api/dpi.svg");
-        RsvgConvert::new_with_input(input)
+    fn accepts_arg(option: &str) {
+        RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
             .arg(option)
             .assert()
             .success();
@@ -77,8 +75,7 @@ impl RsvgConvert {
 
 #[test]
 fn converts_svg_from_stdin_to_png() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .assert()
         .success()
         .stdout(file::is_png());
@@ -86,7 +83,7 @@ fn converts_svg_from_stdin_to_png() {
 
 #[test]
 fn argument_is_input_filename() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
+    let input = Path::new("tests/fixtures/dimensions/521-with-viewbox.svg");
     RsvgConvert::new()
         .arg(input)
         .assert()
@@ -95,49 +92,64 @@ fn argument_is_input_filename() {
 }
 
 #[test]
-fn output_format_png() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
-        .arg("--format=png")
+fn argument_is_url() {
+    let path = Path::new("tests/fixtures/dimensions/521-with-viewbox.svg")
+        .canonicalize()
+        .unwrap();
+    let url = Url::from_file_path(path).unwrap();
+    let stringified = url.as_str();
+    assert!(stringified.starts_with("file://"));
+
+    RsvgConvert::new()
+        .arg(stringified)
         .assert()
         .success()
         .stdout(file::is_png());
 }
 
+#[test]
+fn output_format_png() {
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
+        .arg("--format=png")
+        .assert()
+        .success()
+        .stdout(file::is_png());
+}
+
+#[cfg(have_cairo_ps)]
 #[test]
 fn output_format_ps() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--format=ps")
         .assert()
         .success()
         .stdout(file::is_ps());
 }
 
+#[cfg(have_cairo_ps)]
 #[test]
 fn output_format_eps() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--format=eps")
         .assert()
         .success()
         .stdout(file::is_eps());
 }
 
+#[cfg(have_cairo_pdf)]
 #[test]
 fn output_format_pdf() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--format=pdf")
         .assert()
         .success()
         .stdout(file::is_pdf());
 }
 
+#[cfg(have_cairo_svg)]
 #[test]
 fn output_format_svg_short_option() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("-f")
         .arg("svg")
         .assert()
@@ -153,8 +165,7 @@ fn output_file_option() {
     };
     assert!(predicates::path::is_file().not().eval(&output));
 
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg(format!("--output={}", output.display()))
         .assert()
         .success()
@@ -172,8 +183,7 @@ fn output_file_short_option() {
     };
     assert!(predicates::path::is_file().not().eval(&output));
 
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("-o")
         .arg(format!("{}", output.display()))
         .assert()
@@ -184,20 +194,40 @@ fn output_file_short_option() {
     std::fs::remove_file(&output).unwrap();
 }
 
+#[test]
+fn overwrites_existing_output_file() {
+    let output = {
+        let tempfile = Builder::new().suffix(".png").tempfile().unwrap();
+        tempfile.path().to_path_buf()
+    };
+    assert!(predicates::path::is_file().not().eval(&output));
+
+    for _ in 0..2 {
+        RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
+            .arg(format!("--output={}", output.display()))
+            .assert()
+            .success()
+            .stdout(is_empty());
+
+        assert!(predicates::path::is_file().eval(&output));
+    }
+
+    std::fs::remove_file(&output).unwrap();
+}
+
 #[test]
 fn empty_input_yields_error() {
     let starts_with = starts_with("Error reading SVG");
-    let ends_with = ends_with("Input file is too short");
+    let ends_with = ends_with("Input file is too short").trim();
     RsvgConvert::new()
         .assert()
         .failure()
-        .stderr(starts_with.and(ends_with).trim());
+        .stderr(starts_with.and(ends_with));
 }
 
 #[test]
 fn empty_svg_yields_error() {
-    let input = Path::new("fixtures/dimensions/empty.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/empty.svg")
         .assert()
         .failure()
         .stderr("The SVG stdin has no dimensions\n");
@@ -205,20 +235,23 @@ fn empty_svg_yields_error() {
 
 #[test]
 fn multiple_input_files_not_allowed_for_png_output() {
-    let one = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    let two = Path::new("fixtures/dimensions/sub-rect-no-unit.svg");
+    let one = Path::new("tests/fixtures/dimensions/521-with-viewbox.svg");
+    let two = Path::new("tests/fixtures/dimensions/sub-rect-no-unit.svg");
     RsvgConvert::new()
         .arg(one)
         .arg(two)
         .assert()
         .failure()
-        .stderr("Multiple SVG files are only allowed for PDF and (E)PS output.\n");
+        .stderr(contains(
+            "Multiple SVG files are only allowed for PDF and (E)PS output",
+        ));
 }
 
+#[cfg(have_cairo_ps)]
 #[test]
 fn multiple_input_files_accepted_for_eps_output() {
-    let one = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    let two = Path::new("fixtures/dimensions/sub-rect-no-unit.svg");
+    let one = Path::new("tests/fixtures/dimensions/521-with-viewbox.svg");
+    let two = Path::new("tests/fixtures/dimensions/sub-rect-no-unit.svg");
     RsvgConvert::new()
         .arg("--format=eps")
         .arg(one)
@@ -228,10 +261,11 @@ fn multiple_input_files_accepted_for_eps_output() {
         .stdout(file::is_eps());
 }
 
+#[cfg(have_cairo_ps)]
 #[test]
 fn multiple_input_files_accepted_for_ps_output() {
-    let one = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    let two = Path::new("fixtures/dimensions/sub-rect-no-unit.svg");
+    let one = Path::new("tests/fixtures/dimensions/521-with-viewbox.svg");
+    let two = Path::new("tests/fixtures/dimensions/sub-rect-no-unit.svg");
     RsvgConvert::new()
         .arg("--format=ps")
         .arg(one)
@@ -241,11 +275,12 @@ fn multiple_input_files_accepted_for_ps_output() {
         .stdout(file::is_ps());
 }
 
+#[cfg(have_cairo_pdf)]
 #[test]
 fn multiple_input_files_create_multi_page_pdf_output() {
-    let one = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    let two = Path::new("fixtures/dimensions/sub-rect-no-unit.svg");
-    let three = Path::new("fixtures/api/example.svg");
+    let one = Path::new("tests/fixtures/dimensions/521-with-viewbox.svg");
+    let two = Path::new("tests/fixtures/dimensions/sub-rect-no-unit.svg");
+    let three = Path::new("tests/fixtures/api/example.svg");
     RsvgConvert::new()
         .arg("--format=pdf")
         .arg(one)
@@ -256,9 +291,10 @@ fn multiple_input_files_create_multi_page_pdf_output() {
         .stdout(file::is_pdf().with_page_count(3));
 }
 
+#[cfg(have_cairo_pdf)]
 #[test]
 fn env_source_data_epoch_controls_pdf_creation_date() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
+    let input = Path::new("tests/fixtures/dimensions/521-with-viewbox.svg");
     let date = 1581411039; // seconds since epoch
     RsvgConvert::new()
         .env("SOURCE_DATE_EPOCH", format!("{}", date))
@@ -269,10 +305,11 @@ fn env_source_data_epoch_controls_pdf_creation_date() {
         .stdout(file::is_pdf().with_creation_date(Utc.timestamp(date, 0)));
 }
 
+#[cfg(have_cairo_pdf)]
 #[test]
 fn env_source_data_epoch_no_digits() {
     // intentionally not testing for the full error string here
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
+    let input = Path::new("tests/fixtures/dimensions/521-with-viewbox.svg");
     RsvgConvert::new()
         .env("SOURCE_DATE_EPOCH", "foobar")
         .arg("--format=pdf")
@@ -282,10 +319,11 @@ fn env_source_data_epoch_no_digits() {
         .stderr(starts_with("Environment variable $SOURCE_DATE_EPOCH"));
 }
 
+#[cfg(have_cairo_pdf)]
 #[test]
 fn env_source_data_epoch_trailing_garbage() {
     // intentionally not testing for the full error string here
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
+    let input = Path::new("tests/fixtures/dimensions/521-with-viewbox.svg");
     RsvgConvert::new()
         .arg("--format=pdf")
         .env("SOURCE_DATE_EPOCH", "1234556+")
@@ -295,10 +333,11 @@ fn env_source_data_epoch_trailing_garbage() {
         .stderr(starts_with("Environment variable $SOURCE_DATE_EPOCH"));
 }
 
+#[cfg(have_cairo_pdf)]
 #[test]
 fn env_source_data_epoch_empty() {
     // intentionally not testing for the full error string here
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
+    let input = Path::new("tests/fixtures/dimensions/521-with-viewbox.svg");
     RsvgConvert::new()
         .arg("--format=pdf")
         .env("SOURCE_DATE_EPOCH", "")
@@ -310,8 +349,7 @@ fn env_source_data_epoch_empty() {
 
 #[test]
 fn width_option() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--width=300")
         .assert()
         .success()
@@ -320,8 +358,7 @@ fn width_option() {
 
 #[test]
 fn height_option() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--height=200")
         .assert()
         .success()
@@ -330,8 +367,7 @@ fn height_option() {
 
 #[test]
 fn width_and_height_options() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--width=300")
         .arg("--height=200")
         .assert()
@@ -341,30 +377,16 @@ fn width_and_height_options() {
 
 #[test]
 fn zoom_factor() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--zoom=0.8")
         .assert()
         .success()
         .stdout(file::is_png().with_size(160, 80));
 }
 
-// TODO: Is this a bug in rsvg-convert or the desired behavior ?
-#[test]
-fn zoom_factor_and_width_conflicts() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
-        .arg("--width=400")
-        .arg("--zoom=1.5")
-        .assert()
-        .failure()
-        .stderr(ends_with("Could not render file stdin").trim());
-}
-
 #[test]
 fn zoom_factor_and_larger_size() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--width=400")
         .arg("--height=200")
         .arg("--zoom=1.5")
@@ -375,8 +397,7 @@ fn zoom_factor_and_larger_size() {
 
 #[test]
 fn zoom_factor_and_smaller_size() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--width=400")
         .arg("--height=200")
         .arg("--zoom=3.5")
@@ -387,8 +408,7 @@ fn zoom_factor_and_smaller_size() {
 
 #[test]
 fn x_zoom_option() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--x-zoom=2")
         .assert()
         .success()
@@ -397,8 +417,7 @@ fn x_zoom_option() {
 
 #[test]
 fn x_short_option() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("-x")
         .arg("2.0")
         .assert()
@@ -408,8 +427,7 @@ fn x_short_option() {
 
 #[test]
 fn y_zoom_option() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--y-zoom=2.0")
         .assert()
         .success()
@@ -418,8 +436,7 @@ fn y_zoom_option() {
 
 #[test]
 fn y_short_option() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("-y")
         .arg("2")
         .assert()
@@ -429,71 +446,64 @@ fn y_short_option() {
 
 #[test]
 fn huge_zoom_factor_yields_error() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    let starts_with = starts_with("The resulting image would be larger than 32767 pixels");
-    let ends_with = ends_with("Please specify a smaller size.");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--zoom=1000")
         .assert()
         .failure()
-        .stderr(starts_with.and(ends_with).trim());
+        .stderr(starts_with(
+            "The resulting image would be larger than 32767 pixels",
+        ));
 }
 
 #[test]
-fn default_resolution_is_90dpi() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+fn default_resolution_is_96dpi() {
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .assert()
         .success()
-        .stdout(file::is_png().with_size(90, 360));
+        .stdout(file::is_png().with_size(96, 384));
 }
 
 #[test]
 fn x_resolution() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("--dpi-x=300")
         .assert()
         .success()
-        .stdout(file::is_png().with_size(300, 360));
+        .stdout(file::is_png().with_size(300, 384));
 }
 
 #[test]
 fn x_resolution_short_option() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("-d")
         .arg("45")
         .assert()
         .success()
-        .stdout(file::is_png().with_size(45, 360));
+        .stdout(file::is_png().with_size(45, 384));
 }
 
 #[test]
 fn y_resolution() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("--dpi-y=300")
         .assert()
         .success()
-        .stdout(file::is_png().with_size(90, 1200));
+        .stdout(file::is_png().with_size(96, 1200));
 }
 
 #[test]
 fn y_resolution_short_option() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("-p")
         .arg("45")
         .assert()
         .success()
-        .stdout(file::is_png().with_size(90, 180));
+        .stdout(file::is_png().with_size(96, 180));
 }
 
 #[test]
 fn x_and_y_resolution() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("--dpi-x=300")
         .arg("--dpi-y=150")
         .assert()
@@ -502,31 +512,29 @@ fn x_and_y_resolution() {
 }
 
 #[test]
-fn defaults_are_used_for_zero_resolutions() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+fn zero_resolution_is_invalid() {
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("--dpi-x=0")
         .arg("--dpi-y=0")
         .assert()
-        .success()
-        .stdout(file::is_png().with_size(90, 360));
+        .failure()
+        .stderr(contains("Invalid resolution"));
 }
 
 #[test]
-fn defaults_are_used_for_negative_resolutions() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+fn negative_resolution_is_invalid() {
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("--dpi-x=-100")
         .arg("--dpi-y=-100")
         .assert()
-        .success()
-        .stdout(file::is_png().with_size(90, 360));
+        .failure()
+        .stderr(contains("Invalid resolution"));
 }
 
+#[cfg(have_cairo_pdf)]
 #[test]
 fn pdf_page_size() {
-    let input = Path::new("fixtures/dimensions/521-with-viewbox.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/dimensions/521-with-viewbox.svg")
         .arg("--format=pdf")
         .assert()
         .success()
@@ -537,26 +545,17 @@ fn pdf_page_size() {
 
 #[test]
 fn background_color_option_with_valid_color() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
-        .arg("--background-color=purple")
-        .assert()
-        .success();
+    RsvgConvert::accepts_arg("--background-color=LimeGreen");
 }
 
 #[test]
 fn background_color_option_none() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
-        .arg("--background-color=None")
-        .assert()
-        .success();
+    RsvgConvert::accepts_arg("--background-color=None");
 }
 
 #[test]
 fn background_color_short_option() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("-b")
         .arg("#aabbcc")
         .assert()
@@ -565,37 +564,42 @@ fn background_color_short_option() {
 
 #[test]
 fn background_color_option_invalid_color_yields_error() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("--background-color=foobar")
         .assert()
         .failure()
-        .stderr("Invalid color specification.\n");
+        .stderr(contains("Invalid").and(contains("color")));
+}
+
+#[test]
+fn background_color_is_rendered() {
+    RsvgConvert::new_with_input("tests/fixtures/cmdline/gimp-wilber.svg")
+        .arg("--background-color=purple")
+        .assert()
+        .success()
+        .stdout(file::is_png().with_contents("tests/fixtures/cmdline/gimp-wilber-ref.png"));
 }
 
 #[test]
 fn stylesheet_option() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
-        .arg("--stylesheet=fixtures/dimensions/empty.svg")
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
+        .arg("--stylesheet=tests/fixtures/dimensions/empty.svg")
         .assert()
         .success();
 }
 
 #[test]
 fn stylesheet_short_option() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("-s")
-        .arg("fixtures/dimensions/empty.svg")
+        .arg("tests/fixtures/dimensions/empty.svg")
         .assert()
         .success();
 }
 
 #[test]
 fn stylesheet_option_error() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("--stylesheet=foobar")
         .assert()
         .failure()
@@ -604,46 +608,65 @@ fn stylesheet_option_error() {
 
 #[test]
 fn export_id_option() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
-        .arg("--export-id=one")
+    RsvgConvert::new_with_input("tests/fixtures/api/geometry-element.svg")
+        .arg("--export-id=foo")
         .assert()
-        .success();
+        .success()
+        .stdout(file::is_png().with_size(40, 50));
+}
+
+#[test]
+fn export_id_with_zero_stroke_width() {
+    // https://gitlab.gnome.org/GNOME/librsvg/-/issues/601
+    //
+    // This tests a bug that manifested itself easily with the --export-id option, but it
+    // is not a bug with the option itself.  An object with stroke_width=0 was causing
+    // an extra point at the origin to be put in the bounding box, so the final image
+    // spanned the origin to the actual visible bounds of the rendered object.
+    //
+    // We can probably test this more cleanly once we have a render tree.
+    RsvgConvert::new_with_input("tests/fixtures/cmdline/601-zero-stroke-width.svg")
+        .arg("--export-id=foo")
+        .assert()
+        .success()
+        .stdout(
+            file::is_png()
+                .with_contents("tests/fixtures/cmdline/601-zero-stroke-width-render-only-foo.png"),
+        );
 }
 
 #[test]
 fn export_id_short_option() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("-i")
         .arg("two")
         .assert()
-        .success();
+        .success()
+        .stdout(file::is_png().with_size(100, 200));
 }
 
 #[test]
 fn export_id_option_error() {
-    let input = Path::new("fixtures/api/dpi.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/api/dpi.svg")
         .arg("--export-id=foobar")
         .assert()
         .failure()
-        .stderr("File stdin does not have an object with id \"foobar\"\n");
+        .stderr(starts_with("File stdin does not have an object with id \""));
 }
 
 #[test]
 fn unlimited_option() {
-    RsvgConvert::accepts_option("--unlimited");
+    RsvgConvert::accepts_arg("--unlimited");
 }
 
 #[test]
 fn unlimited_short_option() {
-    RsvgConvert::accepts_option("-u");
+    RsvgConvert::accepts_arg("-u");
 }
 
 #[test]
 fn keep_aspect_ratio_option() {
-    let input = Path::new("fixtures/api/dpi.svg");
+    let input = Path::new("tests/fixtures/api/dpi.svg");
     RsvgConvert::new_with_input(input)
         .arg("--width=500")
         .arg("--height=1000")
@@ -661,7 +684,7 @@ fn keep_aspect_ratio_option() {
 
 #[test]
 fn keep_aspect_ratio_short_option() {
-    let input = Path::new("fixtures/api/dpi.svg");
+    let input = Path::new("tests/fixtures/api/dpi.svg");
     RsvgConvert::new_with_input(input)
         .arg("--width=1000")
         .arg("--height=500")
@@ -679,39 +702,48 @@ fn keep_aspect_ratio_short_option() {
 
 #[test]
 fn overflowing_size_is_detected() {
-    let input = Path::new("fixtures/render-crash/591-vbox-overflow.svg");
-    RsvgConvert::new_with_input(input)
+    RsvgConvert::new_with_input("tests/fixtures/render-crash/591-vbox-overflow.svg")
         .assert()
         .failure()
-        .stderr(contains("Could not get dimensions").trim());
+        .stderr(starts_with(
+            "The resulting image would be larger than 32767 pixels",
+        ));
 }
 
 #[test]
 fn keep_image_data_option() {
-    RsvgConvert::accepts_option("--keep-image-data");
+    RsvgConvert::accepts_arg("--keep-image-data");
 }
 
 #[test]
 fn no_keep_image_data_option() {
-    RsvgConvert::accepts_option("--no-keep-image-data");
+    RsvgConvert::accepts_arg("--no-keep-image-data");
+}
+
+fn is_version_output() -> AndPredicate<StartsWithPredicate, TrimPredicate<EndsWithPredicate>, str> {
+    starts_with("rsvg-convert version ").and(ends_with_pkg_version().trim())
 }
 
 #[test]
 fn version_option() {
-    RsvgConvert::option_yields_output("--version", starts_with("rsvg-convert version "));
+    RsvgConvert::option_yields_output("--version", is_version_output())
 }
 
 #[test]
 fn version_short_option() {
-    RsvgConvert::option_yields_output("-v", starts_with("rsvg-convert version "));
+    RsvgConvert::option_yields_output("-v", is_version_output())
+}
+
+fn is_usage_output() -> OrPredicate<ContainsPredicate, ContainsPredicate, str> {
+    contains("Usage:").or(contains("USAGE:"))
 }
 
 #[test]
 fn help_option() {
-    RsvgConvert::option_yields_output("--help", starts_with("Usage:"));
+    RsvgConvert::option_yields_output("--help", is_usage_output())
 }
 
 #[test]
 fn help_short_option() {
-    RsvgConvert::option_yields_output("-?", starts_with("Usage:"));
+    RsvgConvert::option_yields_output("-?", is_usage_output())
 }
